#include <msp/gl/meshbuilder.h>
#include "geometry.h"
#include "part.h"
#include "partcache.h"
#include "resources.h"

using namespace std;

namespace Msp {
namespace GLtk {

Part::Part(const string &n):
	name(n)
{
	fill(graphic, graphic+N_STATES_, static_cast<Graphic *>(0));
}

const Graphic *Part::get_graphic(State state) const
{
	if(state>N_STATES_)
		throw invalid_argument("Part::get_graphic");

	return graphic[state];
}

void Part::build(const Geometry &parent, State state, PartCache &cache) const
{
	if(!graphic[state] || !graphic[state]->get_texture())
		return;

	Geometry rgeom = geom;
	align.apply(rgeom, parent, margin);
	GL::MeshBuilder bld(cache.create_mesh(*this, *graphic[state]->get_texture()));
	bld.matrix() *= GL::Matrix::translation(rgeom.x, rgeom.y, 0);
	graphic[state]->build(rgeom.w, rgeom.h, bld);
}


Part::Loader::Loader(Part &p, Resources &r):
	DataFile::CollectionObjectLoader<Part>(p, &r)
{
	add("graphic", &Loader::graphic);
	add("align",   &Loader::align);
	add("fill",    &Loader::fill);
	add("margin",  &Loader::margin);
	add("size",    &Loader::size);
}

Part::Loader::~Loader()
{
	for(unsigned i=0; i<N_STATES_; ++i)
		if(obj.graphic[i])
		{
			const Graphic &grph = *obj.graphic[i];
			const Sides &shadow = grph.get_shadow();
			obj.geom.w = max(obj.geom.w, grph.get_width()-shadow.left-shadow.right);
			obj.geom.h = max(obj.geom.h, grph.get_height()-shadow.bottom-shadow.top);
		}
}

void Part::Loader::graphic_normal(const string &n)
{
	graphic(NORMAL, n);
}

void Part::Loader::graphic(State s, const string &n)
{
	Graphic *grph = &get_collection().get<Graphic>(n);
	for(int i=0; i<N_STATES_; ++i)
		if((i&s)==s)
			obj.graphic[i] = grph;
}

void Part::Loader::align(float x, float y)
{
	obj.align.x = x;
	obj.align.y = y;
}

void Part::Loader::fill(float w, float h)
{
	obj.align.w = w;
	obj.align.h = h;
}

void Part::Loader::margin()
{
	load_sub(obj.margin);
}

void Part::Loader::size(unsigned w, unsigned h)
{
	obj.geom.w = w;
	obj.geom.h = h;
}

} // namespace GLtk
} // namespace Msp
